@@ -11,12 +11,16 @@ corrgram(vote, type='corr')
 corrgram(vote, lower.panel=panel.conf)
 test_that("correlation matrix with type='data'", {
   expect_warning(corrgram(vote, type='data'))
+  expect_error(corrgram(vote, type='junk'))
+})
+
+
+test_that("cor.method", {
+  corrgram(auto) # pearson is default
+  corrgram(auto, cor.method="pearson", upper.panel=panel.conf, lower.panel=panel.pie)
+  expect_error(corrgram(auto, cor.method="spearman", upper.panel=panel.conf, lower.panel=panel.pie))
 })
 
-# cor.method
-corrgram(auto)
-corrgram(auto, cor.method="pearson")
-corrgram(auto, cor.method="spearman") # Slight change in colors
 
 # ignore non-numeric columns
 corrgram(iris)
@@ -31,18 +35,31 @@ corrgram(auto, label.srt=-45)
 # label.cex, label.pos
 corrgram(auto, label.srt=45, label.pos=c(.75,.75), cex.labels=2.5, upper=NULL)
 
+test_that("panel correlations", {
+  dat = data.frame(x=c(1,2,NA), y=c(NA,2,3))
+  expect_warning(panel.bar(dat$x, dat$y))
+  expect_warning(panel.conf(dat$x, dat$y))
+  expect_warning(panel.cor(dat$x, dat$y))
+  expect_warning(panel.ellipse(dat$x, dat$y))
+  expect_warning(panel.fill(dat$x, dat$y))
+  expect_warning(panel.pie(dat$x, dat$y))
+  expect_warning(panel.shade(dat$x, dat$y))
+})
 
 # order argument
-corrgram(mtcars)
-corrgram(mtcars, order=NULL)
-corrgram(mtcars, order=FALSE)
-corrgram(mtcars, order=TRUE)
-corrgram(mtcars, order="GW")
-corrgram(mtcars, order="HC")
-corrgram(mtcars, order="PC")
-corrgram(mtcars, order="OLO")
-corrgram(mtcars, order="PC", abs=TRUE)
-corrgram(mtcars, order="OLO", abs=TRUE)
+test_that("order argument", {
+  corrgram(mtcars)
+  corrgram(mtcars, order=NULL)
+  corrgram(mtcars, order=FALSE)
+  corrgram(mtcars, order=TRUE)
+  corrgram(mtcars, order="GW")
+  corrgram(mtcars, order="HC")
+  corrgram(mtcars, order="PC")
+  corrgram(mtcars, order="OLO")
+  corrgram(mtcars, order="PC", abs=TRUE)
+  corrgram(mtcars, order="OLO", abs=TRUE)
+  expect_error(corrgram(mtcars, order="junk", abs=TRUE))
+})
 
 # make sure 'labels' works correctly with 'order'
 myLabels = names(mtcars)
@@ -59,8 +76,12 @@ corrgram(auto, order=TRUE, dir="/")
 # off-diagonal panels
 corrgram(auto, panel=panel.bar)
 corrgram(auto, panel=panel.conf)
+# cex.cor gives warnings "not a graphical parameter"
+#corrgram(auto, panel=panel.conf, cex.cor=1)
 corrgram(auto, panel=panel.cor)
+#corrgram(auto, panel=panel.cor, cex.cor=1.5)
 corrgram(auto, panel=panel.ellipse) # note: latticeExtra also has panel.ellipse
+corrgram(auto, panel=panel.fill)
 corrgram(auto, panel=panel.pie)
 corrgram(auto, panel=panel.pts)
 corrgram(auto, panel=panel.shade)
@@ -76,6 +97,7 @@ corrgram(auto, panel=panel.bar,col.regions=col.earth)
 corrgram(auto, panel=panel.conf,col.regions=col.earth)
 corrgram(auto, panel=panel.cor,col.regions=col.earth)
 corrgram(auto, panel=panel.ellipse,col.regions=col.earth)
+corrgram(auto, panel=panel.fill,col.regions=col.earth)
 corrgram(auto, panel=panel.pie,col.regions=col.earth)
 corrgram(auto, panel=panel.pts,col.regions=col.earth)
 corrgram(auto, panel=panel.shade,col.regions=col.earth)
@@ -173,6 +195,7 @@ if(FALSE){ # No need to test automatically
   corrgram(dati, panel=panel.conf)
   corrgram(dati, panel=panel.cor)
   corrgram(dati, panel=panel.ellipse)
+  corrgram(dati, panel=panel.fill)
   corrgram(dati, panel=panel.pie)
   corrgram(dati, panel=panel.pts)
   corrgram(dati, panel=panel.shade)
